@@ -295,7 +295,7 @@ class StackResource(object):
 class StackResourceSummary(object):
     def __init__(self, connection=None):
         self.connection = connection
-        self.last_updated_timestamp = None
+        self.last_updated_time = None
         self.logical_resource_id = None
         self.physical_resource_id = None
         self.resource_status = None
@@ -306,14 +306,14 @@ class StackResourceSummary(object):
         return None
 
     def endElement(self, name, value, connection):
-        if name == "LastUpdatedTimestamp":
+        if name == "LastUpdatedTime":
             try:
-                self.last_updated_timestamp = datetime.strptime(
+                self.last_updated_time = datetime.strptime(
                     value,
                     '%Y-%m-%dT%H:%M:%SZ'
                 )
             except ValueError:
-                self.last_updated_timestamp = datetime.strptime(
+                self.last_updated_time = datetime.strptime(
                     value,
                     '%Y-%m-%dT%H:%M:%S.%fZ'
                 )
