@@ -35,6 +35,9 @@ def regions():
     return [RegionInfo(name='us-east-1',
                        endpoint='dynamodb.us-east-1.amazonaws.com',
                        connection_cls=DynamoDBConnection),
+            RegionInfo(name='us-gov-west-1',
+                       endpoint='dynamodb.us-gov-west-1.amazonaws.com',
+                       connection_cls=DynamoDBConnection),
             RegionInfo(name='us-west-1',
                        endpoint='dynamodb.us-west-1.amazonaws.com',
                        connection_cls=DynamoDBConnection),
