@@ -21,7 +21,11 @@
 #
 from binascii import crc32
 
-import json
+try:
+    import json
+except ImportError:
+    import simplejson as json
+
 import boto
 from boto.connection import AWSQueryConnection
 from boto.regioninfo import RegionInfo
@@ -31,10 +35,9 @@ from boto.dynamodb2 import exceptions
 
 class DynamoDBConnection(AWSQueryConnection):
     """
-    Amazon DynamoDB is a fast, highly scalable, highly available,
-    cost-effective non-relational database service. Amazon DynamoDB
-    removes traditional scalability limitations on data storage while
-    maintaining low latency and predictable performance.
+    Amazon DynamoDB **Overview**
+    This is the Amazon DynamoDB API Reference. This guide provides
+    descriptions and samples of the Amazon DynamoDB API.
     """
     APIVersion = "2012-08-10"
     DefaultRegionName = "us-east-1"
@@ -67,7 +70,11 @@ class DynamoDBConnection(AWSQueryConnection):
                 if reg.name == region_name:
                     region = reg
                     break
-        kwargs['host'] = region.endpoint
+
+        # Only set host if it isn't manually overwritten
+        if 'host' not in kwargs:
+            kwargs['host'] = region.endpoint
+
         AWSQueryConnection.__init__(self, **kwargs)
         self.region = region
         self._validate_checksums = boto.config.getbool(
@@ -122,7 +129,7 @@ class DynamoDBConnection(AWSQueryConnection):
         result. Requests for nonexistent items consume the minimum
         read capacity units according to the type of read. For more
         information, see `Capacity Units Calculations`_ in the Amazon
-        DynamoDB Developer Guide .
+        DynamoDB Developer Guide.
 
         :type request_items: map
         :param request_items:
@@ -142,7 +149,9 @@ class DynamoDBConnection(AWSQueryConnection):
               `False` (the default), an eventually consistent read is used.
 
         :type return_consumed_capacity: string
-        :param return_consumed_capacity:
+        :param return_consumed_capacity: If set to `TOTAL`, ConsumedCapacity is
+            included in the response; if set to `NONE` (the default),
+            ConsumedCapacity is not included.
 
         """
         params = {'RequestItems': request_items, }
@@ -248,7 +257,9 @@ class DynamoDBConnection(AWSQueryConnection):
                   match those of the schema in the table's attribute definition.
 
         :type return_consumed_capacity: string
-        :param return_consumed_capacity:
+        :param return_consumed_capacity: If set to `TOTAL`, ConsumedCapacity is
+            included in the response; if set to `NONE` (the default),
+            ConsumedCapacity is not included.
 
         :type return_item_collection_metrics: string
         :param return_item_collection_metrics: If set to `SIZE`, statistics
@@ -266,7 +277,8 @@ class DynamoDBConnection(AWSQueryConnection):
                                  body=json.dumps(params))
 
     def create_table(self, attribute_definitions, table_name, key_schema,
-                     provisioned_throughput, local_secondary_indexes=None):
+                     provisioned_throughput, local_secondary_indexes=None,
+                     global_secondary_indexes=None):
         """
         The CreateTable operation adds a new table to your account. In
         an AWS account, table names must be unique within each region.
@@ -298,7 +310,7 @@ class DynamoDBConnection(AWSQueryConnection):
         :param key_schema: Specifies the attributes that make up the primary
             key for the table. The attributes in KeySchema must also be defined
             in the AttributeDefinitions array. For more information, see `Data
-            Model`_ in the Amazon DynamoDB Developer Guide .
+            Model`_ in the Amazon DynamoDB Developer Guide.
         Each KeySchemaElement in the array is composed of:
 
 
@@ -315,7 +327,7 @@ class DynamoDBConnection(AWSQueryConnection):
             KeyType of `RANGE`.
 
         For more information, see `Specifying the Primary Key`_ in the Amazon
-            DynamoDB Developer Guide .
+            DynamoDB Developer Guide.
 
         :type local_secondary_indexes: list
         :param local_secondary_indexes:
@@ -352,8 +364,15 @@ class DynamoDBConnection(AWSQueryConnection):
                   attribute into two different indexes, this counts as two distinct
                   attributes when determining the total.
 
+        :type global_secondary_indexes: list
+        :param global_secondary_indexes:
+
         :type provisioned_throughput: dict
-        :param provisioned_throughput:
+        :param provisioned_throughput: The provisioned throughput settings for
+            the specified table. The settings can be modified using the
+            UpdateTable operation.
+        For current minimum and maximum provisioned throughput values, see
+            `Limits`_ in the Amazon DynamoDB Developer Guide.
 
         """
         params = {
@@ -364,6 +383,8 @@ class DynamoDBConnection(AWSQueryConnection):
         }
         if local_secondary_indexes is not None:
             params['LocalSecondaryIndexes'] = local_secondary_indexes
+        if global_secondary_indexes is not None:
+            params['GlobalSecondaryIndexes'] = global_secondary_indexes
         return self.make_request(action='CreateTable',
                                  body=json.dumps(params))
 
@@ -451,7 +472,9 @@ class DynamoDBConnection(AWSQueryConnection):
         + `ALL_OLD` - The content of the old item is returned.
 
         :type return_consumed_capacity: string
-        :param return_consumed_capacity:
+        :param return_consumed_capacity: If set to `TOTAL`, ConsumedCapacity is
+            included in the response; if set to `NONE` (the default),
+            ConsumedCapacity is not included.
 
         :type return_item_collection_metrics: string
         :param return_item_collection_metrics: If set to `SIZE`, statistics
@@ -488,15 +511,6 @@ class DynamoDBConnection(AWSQueryConnection):
         operations, such as GetItem and PutItem , on a table in the
         `DELETING` state until the table deletion is complete.
 
-        Tables are unique among those associated with the AWS Account
-        issuing the request, and the AWS region that receives the
-        request (such as dynamodb.us-east-1.amazonaws.com). Each
-        Amazon DynamoDB endpoint is entirely independent. For example,
-        if you have two tables called "MyTable," one in dynamodb.us-
-        east-1.amazonaws.com and one in dynamodb.us-
-        west-1.amazonaws.com, they are completely independent and do
-        not share any data; deleting one does not delete the other.
-
         When you delete a table, any local secondary indexes on that
         table are also deleted.
 
@@ -556,7 +570,9 @@ class DynamoDBConnection(AWSQueryConnection):
             are used.
 
         :type return_consumed_capacity: string
-        :param return_consumed_capacity:
+        :param return_consumed_capacity: If set to `TOTAL`, ConsumedCapacity is
+            included in the response; if set to `NONE` (the default),
+            ConsumedCapacity is not included.
 
         """
         params = {'TableName': table_name, 'Key': key, }
@@ -574,14 +590,6 @@ class DynamoDBConnection(AWSQueryConnection):
         Returns an array of all the tables associated with the current
         account and endpoint.
 
-        Each Amazon DynamoDB endpoint is entirely independent. For
-        example, if you have two tables called "MyTable," one in
-        dynamodb.us-east-1.amazonaws.com and one in dynamodb.us-
-        west-1.amazonaws.com , they are completely independent and do
-        not share any data. The ListTables operation returns all of
-        the table names associated with the account making the
-        request, for the endpoint that receives the request.
-
         :type exclusive_start_table_name: string
         :param exclusive_start_table_name: The name of the table that starts
             the list. If you already ran a ListTables operation and received a
@@ -631,7 +639,7 @@ class DynamoDBConnection(AWSQueryConnection):
         primary key attribute, or attributes.
 
         For more information about using this API, see `Working with
-        Items`_ in the Amazon DynamoDB Developer Guide .
+        Items`_ in the Amazon DynamoDB Developer Guide.
 
         :type table_name: string
         :param table_name: The name of the table to contain the item.
@@ -645,7 +653,7 @@ class DynamoDBConnection(AWSQueryConnection):
             the table's attribute definition.
 
         For more information about primary keys, see `Primary Key`_ in the
-            Amazon DynamoDB Developer Guide .
+            Amazon DynamoDB Developer Guide.
 
         Each element in the Item map is an AttributeValue object.
 
@@ -706,7 +714,9 @@ class DynamoDBConnection(AWSQueryConnection):
               the content of the old item is returned.
 
         :type return_consumed_capacity: string
-        :param return_consumed_capacity:
+        :param return_consumed_capacity: If set to `TOTAL`, ConsumedCapacity is
+            included in the response; if set to `NONE` (the default),
+            ConsumedCapacity is not included.
 
         :type return_item_collection_metrics: string
         :param return_item_collection_metrics: If set to `SIZE`, statistics
@@ -826,7 +836,7 @@ class DynamoDBConnection(AWSQueryConnection):
             limit, it stops the operation and returns the matching values up to
             the limit, and a LastEvaluatedKey to apply in a subsequent
             operation to continue the operation. For more information see
-            `Query and Scan`_ in the Amazon DynamoDB Developer Guide .
+            `Query and Scan`_ in the Amazon DynamoDB Developer Guide.
 
         :type consistent_read: boolean
         :param consistent_read: If set to `True`, then the operation uses
@@ -838,7 +848,7 @@ class DynamoDBConnection(AWSQueryConnection):
         The selection criteria for the query.
 
         For a query on a table, you can only have conditions on the table
-            primary key attributes. you must specify the hash key attribute
+            primary key attributes. You must specify the hash key attribute
             name and value as an `EQ` condition. You can optionally specify a
             second condition, referring to the range key attribute.
 
@@ -870,7 +880,7 @@ class DynamoDBConnection(AWSQueryConnection):
               example, equals, greater than, less than, etc. Valid comparison
               operators for Query: `EQ | LE | LT | GE | GT | BEGINS_WITH |
               BETWEEN` For information on specifying data types in JSON, see
-              `JSON Data Format`_ in the Amazon DynamoDB Developer Guide . The
+              `JSON Data Format`_ in the Amazon DynamoDB Developer Guide. The
               following are descriptions of each comparison operator.
 
             + `EQ` : Equal. AttributeValueList can contain only one AttributeValue
@@ -930,18 +940,16 @@ class DynamoDBConnection(AWSQueryConnection):
             ascending order.
 
         :type exclusive_start_key: map
-        :param exclusive_start_key: The primary key of the item from which to
-            continue an earlier operation. An earlier operation might provide
-            this value as the LastEvaluatedKey if that operation was
-            interrupted before completion; either because of the result set
-            size or because of the setting for Limit . The LastEvaluatedKey can
-            be passed back in a new request to continue the operation from that
-            point.
+        :param exclusive_start_key: The primary key of the first item that this
+            operation will evaluate. Use the value that was returned for
+            LastEvaluatedKey in the previous operation.
         The data type for ExclusiveStartKey must be String, Number or Binary.
             No set data types are allowed.
 
         :type return_consumed_capacity: string
-        :param return_consumed_capacity:
+        :param return_consumed_capacity: If set to `TOTAL`, ConsumedCapacity is
+            included in the response; if set to `NONE` (the default),
+            ConsumedCapacity is not included.
 
         """
         params = {'TableName': table_name, }
@@ -986,10 +994,10 @@ class DynamoDBConnection(AWSQueryConnection):
         The result set is eventually consistent.
 
         By default, Scan operations proceed sequentially; however, for
-        faster performance on large tables, applications can perform a
+        faster performance on large tables, applications can request a
         parallel Scan by specifying the Segment and TotalSegments
         parameters. For more information, see `Parallel Scan`_ in the
-        Amazon DynamoDB Developer Guide .
+        Amazon DynamoDB Developer Guide.
 
         :type table_name: string
         :param table_name: The name of the table containing the requested
@@ -1012,7 +1020,7 @@ class DynamoDBConnection(AWSQueryConnection):
             limit, it stops the operation and returns the matching values up to
             the limit, and a LastEvaluatedKey to apply in a subsequent
             operation to continue the operation. For more information see
-            `Query and Scan`_ in the Amazon DynamoDB Developer Guide .
+            `Query and Scan`_ in the Amazon DynamoDB Developer Guide.
 
         :type select: string
         :param select: The attributes to be returned in the result. You can
@@ -1076,7 +1084,7 @@ class DynamoDBConnection(AWSQueryConnection):
               operators for Scan: `EQ | NE | LE | LT | GE | GT | NOT_NULL | NULL
               | CONTAINS | NOT_CONTAINS | BEGINS_WITH | IN | BETWEEN` For
               information on specifying data types in JSON, see `JSON Data
-              Format`_ in the Amazon DynamoDB Developer Guide . The following are
+              Format`_ in the Amazon DynamoDB Developer Guide. The following are
               descriptions of each comparison operator.
 
             + `EQ` : Equal. AttributeValueList can contain only one AttributeValue
@@ -1156,44 +1164,27 @@ class DynamoDBConnection(AWSQueryConnection):
                   "2", "1"]}`
 
         :type exclusive_start_key: map
-        :param exclusive_start_key: The primary key of the item from which to
-            continue an earlier operation. An earlier operation might provide
-            this value as the LastEvaluatedKey if that operation was
-            interrupted before completion; either because of the result set
-            size or because of the setting for Limit . The LastEvaluatedKey can
-            be passed back in a new request to continue the operation from that
-            point.
+        :param exclusive_start_key: The primary key of the first item that this
+            operation will evaluate. Use the value that was returned for
+            LastEvaluatedKey in the previous operation.
         The data type for ExclusiveStartKey must be String, Number or Binary.
             No set data types are allowed.
 
-        If you are performing a parallel scan, the value of ExclusiveStartKey
-            must fall into the key space of the Segment being scanned. For
-            example, suppose that there are two application threads scanning a
-            table using the following Scan parameters
-
-
-        + Thread 0: Segment =0; TotalSegments =2
-        + Thread 1: Segment =1; TotalSegments =2
-
-
-        Now suppose that the Scan request for Thread 0 completed and returned a
-            LastEvaluatedKey of "X". Because "X" is part of Segment 0's key
-            space, it cannot be used anywhere else in the table. If Thread 1
-            were to issue another Scan request with an ExclusiveStartKey of
-            "X", Amazon DynamoDB would throw an InputValidationError because
-            hash key "X" cannot be in Segment 1.
+        In a parallel scan, a Scan request that includes ExclusiveStartKey must
+            specify the same segment whose previous Scan returned the
+            corresponding value of LastEvaluatedKey .
 
         :type return_consumed_capacity: string
-        :param return_consumed_capacity:
+        :param return_consumed_capacity: If set to `TOTAL`, ConsumedCapacity is
+            included in the response; if set to `NONE` (the default),
+            ConsumedCapacity is not included.
 
         :type total_segments: integer
-        :param total_segments: For parallel Scan requests, TotalSegments
-            represents the total number of segments for a table that is being
-            scanned. Segments are a way to logically divide a table into
-            equally sized portions, for the duration of the Scan request. The
-            value of TotalSegments corresponds to the number of application
-            "workers" (such as threads or processes) that will perform the
-            parallel Scan . For example, if you want to scan a table using four
+        :param total_segments: For a parallel Scan request, TotalSegments
+            represents the total number of segments into which the Scan
+            operation will be divided. The value of TotalSegments corresponds
+            to the number of application workers that will perform the parallel
+            scan. For example, if you want to scan a table using four
             application threads, you would specify a TotalSegments value of 4.
         The value for TotalSegments must be greater than or equal to 1, and
             less than or equal to 4096. If you specify a TotalSegments value of
@@ -1202,15 +1193,17 @@ class DynamoDBConnection(AWSQueryConnection):
         If you specify TotalSegments , you must also specify Segment .
 
         :type segment: integer
-        :param segment: For parallel Scan requests, Segment identifies an
-            individual segment to be scanned by an application "worker" (such
-            as a thread or a process). Each worker issues a Scan request with a
-            distinct value for the segment it will scan.
+        :param segment: For a parallel Scan request, Segment identifies an
+            individual segment to be scanned by an application worker.
         Segment IDs are zero-based, so the first segment is always 0. For
             example, if you want to scan a table using four application
             threads, the first thread would specify a Segment value of 0, the
             second thread would specify 1, and so on.
 
+        The value of LastEvaluatedKey returned from a parallel Scan request
+            must be used as ExclusiveStartKey with the same Segment ID in a
+            subsequent Scan operation.
+
         The value for Segment must be greater than or equal to 0, and less than
             the value provided for TotalSegments .
 
@@ -1403,7 +1396,9 @@ class DynamoDBConnection(AWSQueryConnection):
               returned.
 
         :type return_consumed_capacity: string
-        :param return_consumed_capacity:
+        :param return_consumed_capacity: If set to `TOTAL`, ConsumedCapacity is
+            included in the response; if set to `NONE` (the default),
+            ConsumedCapacity is not included.
 
         :type return_item_collection_metrics: string
         :param return_item_collection_metrics: If set to `SIZE`, statistics
@@ -1426,7 +1421,8 @@ class DynamoDBConnection(AWSQueryConnection):
         return self.make_request(action='UpdateItem',
                                  body=json.dumps(params))
 
-    def update_table(self, table_name, provisioned_throughput):
+    def update_table(self, table_name, provisioned_throughput=None,
+                     global_secondary_index_updates=None):
         """
         Updates the provisioned throughput for the given table.
         Setting the throughput for a table helps you manage
@@ -1435,7 +1431,7 @@ class DynamoDBConnection(AWSQueryConnection):
 
         The provisioned throughput values can be upgraded or
         downgraded based on the maximums and minimums listed in the
-        `Limits`_ section in the Amazon DynamoDB Developer Guide .
+        `Limits`_ section in the Amazon DynamoDB Developer Guide.
 
         The table must be in the `ACTIVE` state for this operation to
         succeed. UpdateTable is an asynchronous operation; while
@@ -1454,26 +1450,34 @@ class DynamoDBConnection(AWSQueryConnection):
         :param table_name: The name of the table to be updated.
 
         :type provisioned_throughput: dict
-        :param provisioned_throughput:
+        :param provisioned_throughput: The provisioned throughput settings for
+            the specified table. The settings can be modified using the
+            UpdateTable operation.
+        For current minimum and maximum provisioned throughput values, see
+            `Limits`_ in the Amazon DynamoDB Developer Guide.
+
+        :type global_secondary_index_updates: list
+        :param global_secondary_index_updates:
 
         """
-        params = {
-            'TableName': table_name,
-            'ProvisionedThroughput': provisioned_throughput,
-        }
+        params = {'TableName': table_name, }
+        if provisioned_throughput is not None:
+            params['ProvisionedThroughput'] = provisioned_throughput
+        if global_secondary_index_updates is not None:
+            params['GlobalSecondaryIndexUpdates'] = global_secondary_index_updates
         return self.make_request(action='UpdateTable',
                                  body=json.dumps(params))
 
     def make_request(self, action, body):
         headers = {
             'X-Amz-Target': '%s.%s' % (self.TargetPrefix, action),
-            'Host': self.region.endpoint,
+            'Host': self.host,
             'Content-Type': 'application/x-amz-json-1.0',
             'Content-Length': str(len(body)),
         }
         http_request = self.build_base_http_request(
             method='POST', path='/', auth_path='/', params={},
-            headers=headers, data=body)
+            headers=headers, data=body, host=self.host)
         response = self._mexe(http_request, sender=None,
                               override_num_retries=self.NumberRetries,
                               retry_handler=self._retry_handler)
@@ -1491,6 +1495,7 @@ class DynamoDBConnection(AWSQueryConnection):
 
     def _retry_handler(self, response, i, next_sleep):
         status = None
+        boto.log.debug("Saw HTTP status: %s" % response.status)
         if response.status == 400:
             response_body = response.read()
             boto.log.debug(response_body)
