@@ -1,3 +1,4 @@
+import boto
 from boto.dynamodb2 import exceptions
 from boto.dynamodb2.fields import (HashKey, RangeKey,
                                    AllIndex, KeysOnlyIndex, IncludeIndex)
@@ -57,7 +58,7 @@ class Table(object):
             >>> conn = Table('users')
 
             # The full, minimum-extra-calls case.
-            >>> from boto.dynamodb2.layer1 import DynamoDBConnection
+            >>> from boto import dynamodb2
             >>> users = Table('users', schema=[
             ...     HashKey('username'),
             ...     RangeKey('date_joined', data_type=NUMBER)
@@ -69,11 +70,10 @@ class Table(object):
             ...         RangeKey('date_joined')
             ...     ]),
             ... ],
-            ... connection=DynamoDBConnection(
-            ...     aws_access_key_id='key',
-            ...     aws_secret_access_key='key',
-            ...     region='us-west-2'
-            ... ))
+            ... connection=dynamodb2.connect_to_region('us-west-2',
+		    ...     aws_access_key_id='key',
+		    ...     aws_secret_access_key='key',
+	        ... ))
 
         """
         self.table_name = table_name
@@ -133,7 +133,7 @@ class Table(object):
 
         Example::
 
-            >>> users = Table.create_table('users', schema=[
+            >>> users = Table.create('users', schema=[
             ...     HashKey('username'),
             ...     RangeKey('date_joined', data_type=NUMBER)
             ... ], throughput={
@@ -418,6 +418,45 @@ class Table(object):
         item.load(item_data)
         return item
 
+    def lookup(self, *args, **kwargs):
+        """
+        Look up an entry in DynamoDB. This is mostly backwards compatible
+        with boto.dynamodb. Unlike get_item, it takes hash_key and range_key first,
+        although you may still specify keyword arguments instead.
+
+        Also unlike the get_item command, if the returned item has no keys
+        (i.e., it does not exist in DynamoDB), a None result is returned, instead
+        of an empty key object.
+
+        Example::
+            >>> user = users.lookup(username)
+            >>> user = users.lookup(username, consistent=True)
+            >>> app = apps.lookup('my_customer_id', 'my_app_id')
+
+        """
+        if not self.schema:
+            self.describe()
+        for x, arg in enumerate(args):
+            kwargs[self.schema[x].name] = arg
+        ret = self.get_item(**kwargs)
+        if not ret.keys():
+            return None
+        return ret
+
+    def new_item(self, *args):
+        """
+        Returns a new, blank item
+
+        This is mostly for consistency with boto.dynamodb
+        """
+        if not self.schema:
+            self.describe()
+        data = {}
+        for x, arg in enumerate(args):
+            data[self.schema[x].name] = arg
+        return Item(self, data=data)
+
+
     def put_item(self, data, overwrite=False):
         """
         Saves an entire item to DynamoDB.
@@ -629,6 +668,10 @@ class Table(object):
                     lookup['AttributeValueList'].append(
                         self._dynamizer.encode(value[1])
                     )
+            # Special-case the ``IN`` case
+            elif field_bits[-1] == 'in':
+                for val in value:
+                    lookup['AttributeValueList'].append(self._dynamizer.encode(val))
             else:
                 # Fix up the value for encoding, because it was built to only work
                 # with ``set``s.
@@ -1071,17 +1114,19 @@ class BatchTable(object):
         self.table = table
         self._to_put = []
         self._to_delete = []
+        self._unprocessed = []
 
     def __enter__(self):
         return self
 
     def __exit__(self, type, value, traceback):
-        if not self._to_put and not self._to_delete:
-            return False
+        if self._to_put or self._to_delete:
+            # Flush anything that's left.
+            self.flush()
 
-        # Flush anything that's left.
-        self.flush()
-        return True
+        if self._unprocessed:
+            # Finally, handle anything that wasn't processed.
+            self.resend_unprocessed()
 
     def put_item(self, data, overwrite=False):
         self._to_put.append(data)
@@ -1123,7 +1168,43 @@ class BatchTable(object):
                 }
             })
 
-        self.table.connection.batch_write_item(batch_data)
+        resp = self.table.connection.batch_write_item(batch_data)
+        self.handle_unprocessed(resp)
+
         self._to_put = []
         self._to_delete = []
         return True
+
+    def handle_unprocessed(self, resp):
+        if len(resp.get('UnprocessedItems', [])):
+            table_name = self.table.table_name
+            unprocessed = resp['UnprocessedItems'].get(table_name, [])
+
+            # Some items have not been processed. Stow them for now &
+            # re-attempt processing on ``__exit__``.
+            msg = "%s items were unprocessed. Storing for later."
+            boto.log.info(msg % len(unprocessed))
+            self._unprocessed.extend(unprocessed)
+
+    def resend_unprocessed(self):
+        # If there are unprocessed records (for instance, the user was over
+        # their throughput limitations), iterate over them & send until they're
+        # all there.
+        boto.log.info(
+            "Re-sending %s unprocessed items." % len(self._unprocessed)
+        )
+
+        while len(self._unprocessed):
+            # Again, do 25 at a time.
+            to_resend = self._unprocessed[:25]
+            # Remove them from the list.
+            self._unprocessed = self._unprocessed[25:]
+            batch_data = {
+                self.table.table_name: to_resend
+            }
+            boto.log.info("Sending %s items" % len(to_resend))
+            resp = self.table.connection.batch_write_item(batch_data)
+            self.handle_unprocessed(resp)
+            boto.log.info(
+                "%s unprocessed items left" % len(self._unprocessed)
+            )
