@@ -24,11 +24,7 @@ from datetime import datetime
 from boto.resultset import ResultSet
 from boto.ec2.cloudwatch.listelement import ListElement
 from boto.ec2.cloudwatch.dimension import Dimension
-
-try:
-    import simplejson as json
-except ImportError:
-    import json
+from boto.compat import json
 
 
 class MetricAlarms(list):
@@ -56,10 +52,10 @@ class MetricAlarm(object):
     INSUFFICIENT_DATA = 'INSUFFICIENT_DATA'
 
     _cmp_map = {
-                    '>='    :   'GreaterThanOrEqualToThreshold',
-                    '>'     :   'GreaterThanThreshold',
-                    '<'     :   'LessThanThreshold',
-                    '<='    :   'LessThanOrEqualToThreshold',
+                    '>=': 'GreaterThanOrEqualToThreshold',
+                    '>':  'GreaterThanThreshold',
+                    '<':  'LessThanThreshold',
+                    '<=': 'LessThanOrEqualToThreshold',
                }
     _rev_cmp_map = dict((v, k) for (k, v) in _cmp_map.iteritems())
 
@@ -99,7 +95,7 @@ class MetricAlarm(object):
                        statistic is applied.
 
         :type evaluation_periods: int
-        :param evaluation_period: The number of periods over which data is
+        :param evaluation_periods: The number of periods over which data is
                                   compared to the specified threshold.
 
         :type unit: str
@@ -116,9 +112,16 @@ class MetricAlarm(object):
         :type description: str
         :param description: Description of MetricAlarm
 
-        :type dimensions: list of dicts
-        :param description: Dimensions of alarm, such as:
-                            [{'InstanceId':['i-0123456,i-0123457']}]
+        :type dimensions: dict
+        :param dimensions: A dictionary of dimension key/values where
+                           the key is the dimension name and the value
+                           is either a scalar value or an iterator
+                           of values to be associated with that
+                           dimension.
+                           Example: {
+                               'InstanceId': ['i-0123456', 'i-0123457'],
+                               'LoadBalancerName': 'test-lb'
+                           }
         
         :type alarm_actions: list of strs
         :param alarm_actions: A list of the ARNs of the actions to take in
@@ -290,7 +293,7 @@ class MetricAlarm(object):
         self.ok_actions.append(action_arn)
 
     def delete(self):
-        self.connection.delete_alarms([self])
+        self.connection.delete_alarms([self.name])
 
 class AlarmHistoryItem(object):
     def __init__(self, connection=None):
@@ -312,5 +315,9 @@ class AlarmHistoryItem(object):
         elif name == 'HistorySummary':
             self.summary = value
         elif name == 'Timestamp':
-            self.timestamp = datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%fZ')
+            try:
+                self.timestamp = datetime.strptime(value,
+                                                   '%Y-%m-%dT%H:%M:%S.%fZ')
+            except ValueError:
+                self.timestamp = datetime.strptime(value, '%Y-%m-%dT%H:%M:%SZ')
 
