@@ -482,7 +482,7 @@ class ResumableUploadHandler(object):
         # pool connections) because httplib requires a new HTTP connection per
         # transaction. (Without this, calling http_conn.getresponse() would get
         # "ResponseNotReady".)
-        http_conn = conn.new_http_connection(self.tracker_uri_host,
+        http_conn = conn.new_http_connection(self.tracker_uri_host, conn.port,
                                              conn.is_secure)
         http_conn.set_debuglevel(conn.debug)
 
@@ -664,6 +664,7 @@ class ResumableUploadHandler(object):
                 # Upload succceded, so remove the tracker file (if have one).
                 self._remove_tracker_file()
                 self._check_final_md5(key, etag)
+                key.generation = self.generation
                 if debug >= 1:
                     print 'Resumable upload complete.'
                 return
