@@ -14,7 +14,7 @@
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 # OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
 # ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
-# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
+# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
@@ -30,16 +30,18 @@ import boto.mturk.notification
 from boto.connection import AWSQueryConnection
 from boto.exception import EC2ResponseError
 from boto.resultset import ResultSet
-from boto.mturk.question import QuestionForm, ExternalQuestion
+from boto.mturk.question import QuestionForm, ExternalQuestion, HTMLQuestion
+
 
 class MTurkRequestError(EC2ResponseError):
     "Error for MTurk Requests"
     # todo: subclass from an abstract parent of EC2ResponseError
 
+
 class MTurkConnection(AWSQueryConnection):
-    
-    APIVersion = '2008-08-02'
-    
+
+    APIVersion = '2012-03-25'
+
     def __init__(self, aws_access_key_id=None, aws_secret_access_key=None,
                  is_secure=True, port=None, proxy=None, proxy_port=None,
                  proxy_user=None, proxy_pass=None,
@@ -50,13 +52,14 @@ class MTurkConnection(AWSQueryConnection):
                 host = 'mechanicalturk.sandbox.amazonaws.com'
             else:
                 host = 'mechanicalturk.amazonaws.com'
+        self.debug = debug
 
         AWSQueryConnection.__init__(self, aws_access_key_id,
                                     aws_secret_access_key,
                                     is_secure, port, proxy, proxy_port,
                                     proxy_user, proxy_pass, host, debug,
                                     https_connection_factory)
-    
+
     def _required_auth_capability(self):
         return ['mturk']
 
@@ -67,7 +70,7 @@ class MTurkConnection(AWSQueryConnection):
         return self._process_request('GetAccountBalance', params,
                                      [('AvailableBalance', Price),
                                       ('OnHoldBalance', Price)])
-    
+
     def register_hit_type(self, title, description, reward, duration,
                           keywords=None, approval_delay=None, qual_req=None):
         """
@@ -79,8 +82,7 @@ class MTurkConnection(AWSQueryConnection):
         params = dict(
             Title=title,
             Description=description,
-            AssignmentDurationInSeconds=
-                self.duration_as_seconds(duration),
+            AssignmentDurationInSeconds=self.duration_as_seconds(duration),
             )
         params.update(MTurkConnection.get_price_as_price(reward).get_as_params('Reward'))
 
@@ -94,31 +96,55 @@ class MTurkConnection(AWSQueryConnection):
         if qual_req is not None:
             params.update(qual_req.get_as_params())
 
-        return self._process_request('RegisterHITType', params)
+        return self._process_request('RegisterHITType', params,
+                                     [('HITTypeId', HITTypeId)])
 
     def set_email_notification(self, hit_type, email, event_types=None):
         """
         Performs a SetHITTypeNotification operation to set email
         notification for a specified HIT type
         """
-        return self._set_notification(hit_type, 'Email', email, event_types)
-    
+        return self._set_notification(hit_type, 'Email', email,
+                                      'SetHITTypeNotification', event_types)
+
     def set_rest_notification(self, hit_type, url, event_types=None):
         """
         Performs a SetHITTypeNotification operation to set REST notification
         for a specified HIT type
         """
-        return self._set_notification(hit_type, 'REST', url, event_types)
-        
-    def _set_notification(self, hit_type, transport, destination, event_types=None):
+        return self._set_notification(hit_type, 'REST', url,
+                                      'SetHITTypeNotification', event_types)
+
+    def set_sqs_notification(self, hit_type, queue_url, event_types=None):
         """
-        Common SetHITTypeNotification operation to set notification for a
-        specified HIT type
+        Performs a SetHITTypeNotification operation so set SQS notification
+        for a specified HIT type. Queue URL is of form:
+        https://queue.amazonaws.com/<CUSTOMER_ID>/<QUEUE_NAME> and can be
+        found when looking at the details for a Queue in the AWS Console
+        """
+        return self._set_notification(hit_type, "SQS", queue_url,
+                                      'SetHITTypeNotification', event_types)
+
+    def send_test_event_notification(self, hit_type, url,
+                                     event_types=None,
+                                     test_event_type='Ping'):
+        """
+        Performs a SendTestEventNotification operation with REST notification
+        for a specified HIT type
+        """
+        return self._set_notification(hit_type, 'REST', url,
+                                      'SendTestEventNotification',
+                                      event_types, test_event_type)
+
+    def _set_notification(self, hit_type, transport,
+                          destination, request_type,
+                          event_types=None, test_event_type=None):
+        """
+        Common operation to set notification or send a test event
+        notification for a specified HIT type
         """
-        assert type(hit_type) is str, "hit_type argument should be a string."
-        
         params = {'HITTypeId': hit_type}
-        
+
         # from the Developer Guide:
         # The 'Active' parameter is optional. If omitted, the active status of
         # the HIT type's notification specification is unchanged. All HIT types
@@ -131,54 +157,67 @@ class MTurkConnection(AWSQueryConnection):
 
         # add specific event types if required
         if event_types:
-            self.build_list_params(notification_params, event_types, 'EventType')
-        
+            self.build_list_params(notification_params, event_types,
+                                   'EventType')
+
         # Set up dict of 'Notification.1.Transport' etc. values
         notification_rest_params = {}
         num = 1
         for key in notification_params:
             notification_rest_params['Notification.%d.%s' % (num, key)] = notification_params[key]
-        
+
         # Update main params dict
         params.update(notification_rest_params)
-        
+
+        # If test notification, specify the notification type to be tested
+        if test_event_type:
+            params.update({'TestEventType': test_event_type})
+
         # Execute operation
-        return self._process_request('SetHITTypeNotification', params)
-    
-    def create_hit(self, hit_type=None, question=None,
+        return self._process_request(request_type, params)
+
+    def create_hit(self, hit_type=None, question=None, hit_layout=None,
                    lifetime=datetime.timedelta(days=7),
-                   max_assignments=1, 
+                   max_assignments=1,
                    title=None, description=None, keywords=None,
                    reward=None, duration=datetime.timedelta(days=7),
                    approval_delay=None, annotation=None,
                    questions=None, qualifications=None,
-                   response_groups=None):
+                   layout_params=None, response_groups=None):
         """
         Creates a new HIT.
         Returns a ResultSet
-        See: http://docs.amazonwebservices.com/AWSMechanicalTurkRequester/2006-10-31/ApiReference_CreateHITOperation.html
+        See: http://docs.amazonwebservices.com/AWSMechTurk/2012-03-25/AWSMturkAPI/ApiReference_CreateHITOperation.html
         """
-        
-        # handle single or multiple questions
-        neither = question is None and questions is None
-        both = question is not None and questions is not None
-        if neither or both:
-            raise ValueError("Must specify either question (single Question instance) or questions (list or QuestionForm instance), but not both")
 
-        if question:
-            questions = [question]
-        question_param = QuestionForm(questions)
-        if isinstance(question, QuestionForm):
-            question_param = question
-        elif isinstance(question, ExternalQuestion):
-            question_param = question
-        
         # Handle basic required arguments and set up params dict
-        params = {'Question': question_param.get_as_xml(),
-                  'LifetimeInSeconds' :
+        params = {'LifetimeInSeconds':
                       self.duration_as_seconds(lifetime),
-                  'MaxAssignments' : max_assignments,
-                  }
+                  'MaxAssignments': max_assignments,
+                 }
+
+        # handle single or multiple questions or layouts
+        neither = question is None and questions is None
+        if hit_layout is None:
+            both = question is not None and questions is not None
+            if neither or both:
+                raise ValueError("Must specify question (single Question instance) or questions (list or QuestionForm instance), but not both")
+            if question:
+                questions = [question]
+            question_param = QuestionForm(questions)
+            if isinstance(question, QuestionForm):
+                question_param = question
+            elif isinstance(question, ExternalQuestion):
+                question_param = question
+            elif isinstance(question, HTMLQuestion):
+                question_param = question
+            params['Question'] = question_param.get_as_xml()
+        else:
+            if not neither:
+                raise ValueError("Must not specify question (single Question instance) or questions (list or QuestionForm instance) when specifying hit_layout")
+            params['HITLayoutId'] = hit_layout
+            if layout_params:
+                params.update(layout_params.get_as_params())
 
         # if hit type specified then add it
         # else add the additional required parameters
@@ -187,10 +226,10 @@ class MTurkConnection(AWSQueryConnection):
         else:
             # Handle keywords
             final_keywords = MTurkConnection.get_keywords_as_string(keywords)
-            
+
             # Handle price argument
             final_price = MTurkConnection.get_price_as_price(reward)
-            
+
             final_duration = self.duration_as_seconds(duration)
 
             additional_params = dict(
@@ -211,7 +250,7 @@ class MTurkConnection(AWSQueryConnection):
         # add the annotation if specified
         if annotation is not None:
             params['RequesterAnnotation'] = annotation
-               
+
         # Add the Qualifications if specified
         if qualifications is not None:
             params.update(qualifications.get_as_params())
@@ -219,43 +258,44 @@ class MTurkConnection(AWSQueryConnection):
         # Handle optional response groups argument
         if response_groups:
             self.build_list_params(params, response_groups, 'ResponseGroup')
-                
+
         # Submit
-        return self._process_request('CreateHIT', params, [('HIT', HIT),])
+        return self._process_request('CreateHIT', params, [('HIT', HIT)])
 
     def change_hit_type_of_hit(self, hit_id, hit_type):
         """
         Change the HIT type of an existing HIT. Note that the reward associated
         with the new HIT type must match the reward of the current HIT type in
         order for the operation to be valid.
-        
+
         :type hit_id: str
         :type hit_type: str
         """
-        params = {'HITId' : hit_id,
+        params = {'HITId': hit_id,
                   'HITTypeId': hit_type}
 
         return self._process_request('ChangeHITTypeOfHIT', params)
-    
+
     def get_reviewable_hits(self, hit_type=None, status='Reviewable',
-                            sort_by='Expiration', sort_direction='Ascending', 
+                            sort_by='Expiration', sort_direction='Ascending',
                             page_size=10, page_number=1):
         """
         Retrieve the HITs that have a status of Reviewable, or HITs that
         have a status of Reviewing, and that belong to the Requester
         calling the operation.
         """
-        params = {'Status' : status,
-                  'SortProperty' : sort_by,
-                  'SortDirection' : sort_direction,
-                  'PageSize' : page_size,
-                  'PageNumber' : page_number}
+        params = {'Status': status,
+                  'SortProperty': sort_by,
+                  'SortDirection': sort_direction,
+                  'PageSize': page_size,
+                  'PageNumber': page_number}
 
         # Handle optional hit_type argument
         if hit_type is not None:
             params.update({'HITTypeId': hit_type})
 
-        return self._process_request('GetReviewableHITs', params, [('HIT', HIT),])
+        return self._process_request('GetReviewableHITs', params,
+                                     [('HIT', HIT)])
 
     @staticmethod
     def _get_pages(page_size, total_records):
@@ -263,14 +303,13 @@ class MTurkConnection(AWSQueryConnection):
         Given a page size (records per page) and a total number of
         records, return the page numbers to be retrieved.
         """
-        pages = total_records/page_size+bool(total_records%page_size)
-        return range(1, pages+1)
-
+        pages = total_records / page_size + bool(total_records % page_size)
+        return range(1, pages + 1)
 
     def get_all_hits(self):
         """
         Return all of a Requester's HITs
-        
+
         Despite what search_hits says, it does not return all hits, but
         instead returns a page of hits. This method will pull the hits
         from the server 100 at a time, but will yield the results
@@ -279,12 +318,12 @@ class MTurkConnection(AWSQueryConnection):
         page_size = 100
         search_rs = self.search_hits(page_size=page_size)
         total_records = int(search_rs.TotalNumResults)
-        get_page_hits = lambda(page): self.search_hits(page_size=page_size, page_number=page)
+        get_page_hits = lambda page: self.search_hits(page_size=page_size, page_number=page)
         page_nums = self._get_pages(page_size, total_records)
         hit_sets = itertools.imap(get_page_hits, page_nums)
         return itertools.chain.from_iterable(hit_sets)
 
-    def search_hits(self, sort_by='CreationTime', sort_direction='Ascending', 
+    def search_hits(self, sort_by='CreationTime', sort_direction='Ascending',
                     page_size=10, page_number=1, response_groups=None):
         """
         Return a page of a Requester's HITs, on behalf of the Requester.
@@ -294,22 +333,50 @@ class MTurkConnection(AWSQueryConnection):
         The SearchHITs operation does not accept any search parameters
         that filter the results.
         """
-        params = {'SortProperty' : sort_by,
-                  'SortDirection' : sort_direction,
-                  'PageSize' : page_size,
-                  'PageNumber' : page_number}
+        params = {'SortProperty': sort_by,
+                  'SortDirection': sort_direction,
+                  'PageSize': page_size,
+                  'PageNumber': page_number}
         # Handle optional response groups argument
         if response_groups:
             self.build_list_params(params, response_groups, 'ResponseGroup')
-                
 
-        return self._process_request('SearchHITs', params, [('HIT', HIT),])
+        return self._process_request('SearchHITs', params, [('HIT', HIT)])
+
+    def get_assignment(self, assignment_id, response_groups=None):
+        """
+        Retrieves an assignment using the assignment's ID. Requesters can only
+        retrieve their own assignments, and only assignments whose related HIT
+        has not been disposed.
+
+        The returned ResultSet will have the following attributes:
+
+        Request
+                This element is present only if the Request ResponseGroup
+                is specified.
+        Assignment
+                The assignment. The response includes one Assignment object.
+        HIT
+                The HIT associated with this assignment. The response
+                includes one HIT object.
+
+        """
+
+        params = {'AssignmentId': assignment_id}
+
+        # Handle optional response groups argument
+        if response_groups:
+            self.build_list_params(params, response_groups, 'ResponseGroup')
+
+        return self._process_request('GetAssignment', params,
+                                     [('Assignment', Assignment),
+                                      ('HIT', HIT)])
 
     def get_assignments(self, hit_id, status=None,
-                            sort_by='SubmitTime', sort_direction='Ascending', 
+                            sort_by='SubmitTime', sort_direction='Ascending',
                             page_size=10, page_number=1, response_groups=None):
         """
-        Retrieves completed assignments for a HIT. 
+        Retrieves completed assignments for a HIT.
         Use this operation to retrieve the results for a HIT.
 
         The returned ResultSet will have the following attributes:
@@ -328,14 +395,14 @@ class MTurkConnection(AWSQueryConnection):
                 on this call.
                 A non-negative integer
 
-        The ResultSet will contain zero or more Assignment objects 
+        The ResultSet will contain zero or more Assignment objects
 
         """
-        params = {'HITId' : hit_id,
-                  'SortProperty' : sort_by,
-                  'SortDirection' : sort_direction,
-                  'PageSize' : page_size,
-                  'PageNumber' : page_number}
+        params = {'HITId': hit_id,
+                  'SortProperty': sort_by,
+                  'SortDirection': sort_direction,
+                  'PageSize': page_size,
+                  'PageNumber': page_number}
 
         if status is not None:
             params['AssignmentStatus'] = status
@@ -343,14 +410,14 @@ class MTurkConnection(AWSQueryConnection):
         # Handle optional response groups argument
         if response_groups:
             self.build_list_params(params, response_groups, 'ResponseGroup')
-                
+
         return self._process_request('GetAssignmentsForHIT', params,
-                                     [('Assignment', Assignment),])
+                                     [('Assignment', Assignment)])
 
     def approve_assignment(self, assignment_id, feedback=None):
         """
         """
-        params = {'AssignmentId' : assignment_id,}
+        params = {'AssignmentId': assignment_id}
         if feedback:
             params['RequesterFeedback'] = feedback
         return self._process_request('ApproveAssignment', params)
@@ -358,31 +425,39 @@ class MTurkConnection(AWSQueryConnection):
     def reject_assignment(self, assignment_id, feedback=None):
         """
         """
-        params = {'AssignmentId' : assignment_id,}
+        params = {'AssignmentId': assignment_id}
         if feedback:
             params['RequesterFeedback'] = feedback
         return self._process_request('RejectAssignment', params)
 
+    def approve_rejected_assignment(self, assignment_id, feedback=None):
+        """
+        """
+        params = {'AssignmentId': assignment_id}
+        if feedback:
+            params['RequesterFeedback'] = feedback
+        return self._process_request('ApproveRejectedAssignment', params)
+
     def get_hit(self, hit_id, response_groups=None):
         """
         """
-        params = {'HITId' : hit_id,}
+        params = {'HITId': hit_id}
         # Handle optional response groups argument
         if response_groups:
             self.build_list_params(params, response_groups, 'ResponseGroup')
-                
-        return self._process_request('GetHIT', params, [('HIT', HIT),])
+
+        return self._process_request('GetHIT', params, [('HIT', HIT)])
 
     def set_reviewing(self, hit_id, revert=None):
         """
-        Update a HIT with a status of Reviewable to have a status of Reviewing, 
+        Update a HIT with a status of Reviewable to have a status of Reviewing,
         or reverts a Reviewing HIT back to the Reviewable status.
 
         Only HITs with a status of Reviewable can be updated with a status of
         Reviewing.  Similarly, only Reviewing HITs can be reverted back to a
         status of Reviewable.
         """
-        params = {'HITId' : hit_id,}
+        params = {'HITId': hit_id}
         if revert:
             params['Revert'] = revert
         return self._process_request('SetHITAsReviewing', params)
@@ -404,11 +479,11 @@ class MTurkConnection(AWSQueryConnection):
         It is not possible to re-enable a HIT once it has been disabled.
         To make the work from a disabled HIT available again, create a new HIT.
         """
-        params = {'HITId' : hit_id,}
+        params = {'HITId': hit_id}
         # Handle optional response groups argument
         if response_groups:
             self.build_list_params(params, response_groups, 'ResponseGroup')
-                
+
         return self._process_request('DisableHIT', params)
 
     def dispose_hit(self, hit_id):
@@ -421,7 +496,7 @@ class MTurkConnection(AWSQueryConnection):
         reviewable, then call GetAssignmentsForHIT to retrieve the
         assignments.  Disposing of a HIT removes the HIT from the
         results of a call to GetReviewableHITs.  """
-        params = {'HITId' : hit_id,}
+        params = {'HITId': hit_id}
         return self._process_request('DisposeHIT', params)
 
     def expire_hit(self, hit_id):
@@ -438,14 +513,15 @@ class MTurkConnection(AWSQueryConnection):
         submitted, the expired HIT becomes"reviewable", and will be
         returned by a call to GetReviewableHITs.
         """
-        params = {'HITId' : hit_id,}
+        params = {'HITId': hit_id}
         return self._process_request('ForceExpireHIT', params)
 
-    def extend_hit(self, hit_id, assignments_increment=None, expiration_increment=None):
+    def extend_hit(self, hit_id, assignments_increment=None,
+                   expiration_increment=None):
         """
         Increase the maximum number of assignments, or extend the
         expiration date, of an existing HIT.
-        
+
         NOTE: If a HIT has a status of Reviewable and the HIT is
         extended to make it Available, the HIT will not be returned by
         GetReviewableHITs, and its submitted assignments will not be
@@ -460,7 +536,7 @@ class MTurkConnection(AWSQueryConnection):
            (assignments_increment is not None and expiration_increment is not None):
             raise ValueError("Must specify either assignments_increment or expiration_increment, but not both")
 
-        params = {'HITId' : hit_id,}
+        params = {'HITId': hit_id}
         if assignments_increment:
             params['MaxAssignmentsIncrement'] = assignments_increment
         if expiration_increment:
@@ -476,7 +552,7 @@ class MTurkConnection(AWSQueryConnection):
 
         help_type: either 'Operation' or 'ResponseGroup'
         """
-        params = {'About': about, 'HelpType': help_type,}
+        params = {'About': about, 'HelpType': help_type}
         return self._process_request('Help', params)
 
     def grant_bonus(self, worker_id, assignment_id, bonus_price, reason):
@@ -511,12 +587,12 @@ class MTurkConnection(AWSQueryConnection):
         params = {'WorkerId': worker_id, 'Reason': reason}
 
         return self._process_request('UnblockWorker', params)
-    
+
     def notify_workers(self, worker_ids, subject, message_text):
         """
         Send a text message to workers.
         """
-        params = {'Subject' : subject,
+        params = {'Subject': subject,
                   'MessageText': message_text}
         self.build_list_params(params, worker_ids, 'WorkerId')
 
@@ -567,9 +643,9 @@ class MTurkConnection(AWSQueryConnection):
 
         """
 
-        params = {'Name' : name,
-                  'Description' : description,
-                  'QualificationTypeStatus' : status,
+        params = {'Name': name,
+                  'Description': description,
+                  'QualificationTypeStatus': status,
                   }
         if retry_delay is not None:
             params['RetryDelayInSeconds'] = retry_delay
@@ -584,7 +660,7 @@ class MTurkConnection(AWSQueryConnection):
 
         if answer_key is not None:
             if isinstance(answer_key, basestring):
-                params['AnswerKey'] = answer_key # xml
+                params['AnswerKey'] = answer_key  # xml
             else:
                 raise TypeError
                 # Eventually someone will write an AnswerKey class.
@@ -598,17 +674,29 @@ class MTurkConnection(AWSQueryConnection):
             params['Keywords'] = self.get_keywords_as_string(keywords)
 
         return self._process_request('CreateQualificationType', params,
-                                     [('QualificationType', QualificationType),])
+                                     [('QualificationType',
+                                       QualificationType)])
 
     def get_qualification_type(self, qualification_type_id):
-        params = {'QualificationTypeId' : qualification_type_id }
+        params = {'QualificationTypeId': qualification_type_id }
         return self._process_request('GetQualificationType', params,
-                                     [('QualificationType', QualificationType),])
+                                     [('QualificationType', QualificationType)])
 
-    def get_qualifications_for_qualification_type(self, qualification_type_id):
-        params = {'QualificationTypeId' : qualification_type_id }
+    def get_all_qualifications_for_qual_type(self, qualification_type_id):
+        page_size = 100
+        search_qual = self.get_qualifications_for_qualification_type(qualification_type_id)
+        total_records = int(search_qual.TotalNumResults)
+        get_page_quals = lambda page: self.get_qualifications_for_qualification_type(qualification_type_id = qualification_type_id, page_size=page_size, page_number = page)
+        page_nums = self._get_pages(page_size, total_records)
+        qual_sets = itertools.imap(get_page_quals, page_nums)
+        return itertools.chain.from_iterable(qual_sets)
+
+    def get_qualifications_for_qualification_type(self, qualification_type_id, page_size=100, page_number = 1):
+        params = {'QualificationTypeId': qualification_type_id,
+                  'PageSize': page_size,
+                  'PageNumber': page_number}
         return self._process_request('GetQualificationsForQualificationType', params,
-                                     [('QualificationType', QualificationType),])
+                                     [('Qualification', Qualification)])
 
     def update_qualification_type(self, qualification_type_id,
                                   description=None,
@@ -620,7 +708,7 @@ class MTurkConnection(AWSQueryConnection):
                                   auto_granted=None,
                                   auto_granted_value=None):
 
-        params = {'QualificationTypeId' : qualification_type_id }
+        params = {'QualificationTypeId': qualification_type_id}
 
         if description is not None:
             params['Description'] = description
@@ -640,7 +728,7 @@ class MTurkConnection(AWSQueryConnection):
 
         if answer_key is not None:
             if isinstance(answer_key, basestring):
-                params['AnswerKey'] = answer_key # xml
+                params['AnswerKey'] = answer_key  # xml
             else:
                 raise TypeError
                 # Eventually someone will write an AnswerKey class.
@@ -652,11 +740,11 @@ class MTurkConnection(AWSQueryConnection):
             params['AutoGrantedValue'] = auto_granted_value
 
         return self._process_request('UpdateQualificationType', params,
-                                     [('QualificationType', QualificationType),])
+                                     [('QualificationType', QualificationType)])
 
     def dispose_qualification_type(self, qualification_type_id):
         """TODO: Document."""
-        params = {'QualificationTypeId' : qualification_type_id}
+        params = {'QualificationTypeId': qualification_type_id}
         return self._process_request('DisposeQualificationType', params)
 
     def search_qualification_types(self, query=None, sort_by='Name',
@@ -664,46 +752,46 @@ class MTurkConnection(AWSQueryConnection):
                                    page_number=1, must_be_requestable=True,
                                    must_be_owned_by_caller=True):
         """TODO: Document."""
-        params = {'Query' : query,
-                  'SortProperty' : sort_by,
-                  'SortDirection' : sort_direction,
-                  'PageSize' : page_size,
-                  'PageNumber' : page_number,
-                  'MustBeRequestable' : must_be_requestable,
-                  'MustBeOwnedByCaller' : must_be_owned_by_caller}
+        params = {'Query': query,
+                  'SortProperty': sort_by,
+                  'SortDirection': sort_direction,
+                  'PageSize': page_size,
+                  'PageNumber': page_number,
+                  'MustBeRequestable': must_be_requestable,
+                  'MustBeOwnedByCaller': must_be_owned_by_caller}
         return self._process_request('SearchQualificationTypes', params,
-                    [('QualificationType', QualificationType),])
+                    [('QualificationType', QualificationType)])
 
     def get_qualification_requests(self, qualification_type_id,
                                    sort_by='Expiration',
                                    sort_direction='Ascending', page_size=10,
                                    page_number=1):
         """TODO: Document."""
-        params = {'QualificationTypeId' : qualification_type_id,
-                  'SortProperty' : sort_by,
-                  'SortDirection' : sort_direction,
-                  'PageSize' : page_size,
-                  'PageNumber' : page_number}
+        params = {'QualificationTypeId': qualification_type_id,
+                  'SortProperty': sort_by,
+                  'SortDirection': sort_direction,
+                  'PageSize': page_size,
+                  'PageNumber': page_number}
         return self._process_request('GetQualificationRequests', params,
-                    [('QualificationRequest', QualificationRequest),])
+                    [('QualificationRequest', QualificationRequest)])
 
     def grant_qualification(self, qualification_request_id, integer_value=1):
         """TODO: Document."""
-        params = {'QualificationRequestId' : qualification_request_id,
-                  'IntegerValue' : integer_value}
+        params = {'QualificationRequestId': qualification_request_id,
+                  'IntegerValue': integer_value}
         return self._process_request('GrantQualification', params)
 
     def revoke_qualification(self, subject_id, qualification_type_id,
                              reason=None):
         """TODO: Document."""
-        params = {'SubjectId' : subject_id,
-                  'QualificationTypeId' : qualification_type_id,
-                  'Reason' : reason}
+        params = {'SubjectId': subject_id,
+                  'QualificationTypeId': qualification_type_id,
+                  'Reason': reason}
         return self._process_request('RevokeQualification', params)
 
     def assign_qualification(self, qualification_type_id, worker_id,
                              value=1, send_notification=True):
-        params = {'QualificationTypeId' : qualification_type_id,
+        params = {'QualificationTypeId': qualification_type_id,
                   'WorkerId' : worker_id,
                   'IntegerValue' : value,
                   'SendNotification' : send_notification}
@@ -714,7 +802,7 @@ class MTurkConnection(AWSQueryConnection):
         params = {'QualificationTypeId' : qualification_type_id,
                   'SubjectId' : worker_id}
         return self._process_request('GetQualificationScore', params,
-                    [('Qualification', Qualification),])
+                    [('Qualification', Qualification)])
 
     def update_qualification_score(self, qualification_type_id, worker_id,
                                    value):
@@ -728,7 +816,8 @@ class MTurkConnection(AWSQueryConnection):
         """
         Helper to process the xml response from AWS
         """
-        response = self.make_request(request_type, params, verb='POST')
+        params['Operation'] = request_type
+        response = self.make_request(None, params, verb='POST')
         return self._process_response(response, marker_elems)
 
     def _process_response(self, response, marker_elems=None):
@@ -736,7 +825,8 @@ class MTurkConnection(AWSQueryConnection):
         Helper to process the xml response from AWS
         """
         body = response.read()
-        #print body
+        if self.debug == 2:
+            print body
         if '<Errors>' not in body:
             rs = ResultSet(marker_elems)
             h = handler.XmlHandler(rs, self)
@@ -751,18 +841,18 @@ class MTurkConnection(AWSQueryConnection):
         Returns a comma+space-separated string of keywords from either
         a list or a string
         """
-        if type(keywords) is list:
+        if isinstance(keywords, list):
             keywords = ', '.join(keywords)
-        if type(keywords) is str:
+        if isinstance(keywords, str):
             final_keywords = keywords
-        elif type(keywords) is unicode:
+        elif isinstance(keywords, unicode):
             final_keywords = keywords.encode('utf-8')
         elif keywords is None:
             final_keywords = ""
         else:
             raise TypeError("keywords argument must be a string or a list of strings; got a %s" % type(keywords))
         return final_keywords
-    
+
     @staticmethod
     def get_price_as_price(reward):
         """
@@ -777,13 +867,14 @@ class MTurkConnection(AWSQueryConnection):
     @staticmethod
     def duration_as_seconds(duration):
         if isinstance(duration, datetime.timedelta):
-            duration = duration.days*86400 + duration.seconds
+            duration = duration.days * 86400 + duration.seconds
         try:
             duration = int(duration)
         except TypeError:
             raise TypeError("Duration must be a timedelta or int-castable, got %s" % type(duration))
         return duration
 
+
 class BaseAutoResultElement:
     """
     Base class to automatically add attributes when parsing XML
@@ -797,11 +888,12 @@ class BaseAutoResultElement:
     def endElement(self, name, value, connection):
         setattr(self, name, value)
 
+
 class HIT(BaseAutoResultElement):
     """
     Class to extract a HIT structure from a response (used in ResultSet)
-    
-    Will have attributes named as per the Developer Guide, 
+
+    Will have attributes named as per the Developer Guide,
     e.g. HITId, HITTypeId, CreationTime
     """
 
@@ -820,48 +912,70 @@ class HIT(BaseAutoResultElement):
     # are we there yet?
     expired = property(_has_expired)
 
+
+class HITTypeId(BaseAutoResultElement):
+    """
+    Class to extract an HITTypeId structure from a response
+    """
+
+    pass
+
+
 class Qualification(BaseAutoResultElement):
     """
     Class to extract an Qualification structure from a response (used in
     ResultSet)
-    
+
     Will have attributes named as per the Developer Guide such as
     QualificationTypeId, IntegerValue. Does not seem to contain GrantTime.
     """
-    
+
     pass
 
+
 class QualificationType(BaseAutoResultElement):
     """
     Class to extract an QualificationType structure from a response (used in
     ResultSet)
-    
-    Will have attributes named as per the Developer Guide, 
+
+    Will have attributes named as per the Developer Guide,
     e.g. QualificationTypeId, CreationTime, Name, etc
     """
-    
+
     pass
 
+
 class QualificationRequest(BaseAutoResultElement):
     """
     Class to extract an QualificationRequest structure from a response (used in
     ResultSet)
-    
-    Will have attributes named as per the Developer Guide, 
-    e.g. QualificationRequestId, QualificationTypeId, SubjectId, etc
 
-    TODO: Ensure that Test and Answer attribute are treated properly if the
-          qualification requires a test. These attributes are XML-encoded.
+    Will have attributes named as per the Developer Guide,
+    e.g. QualificationRequestId, QualificationTypeId, SubjectId, etc
     """
-    
-    pass
+
+    def __init__(self, connection):
+        BaseAutoResultElement.__init__(self, connection)
+        self.answers = []
+
+    def endElement(self, name, value, connection):
+        # the answer consists of embedded XML, so it needs to be parsed independantly
+        if name == 'Answer':
+            answer_rs = ResultSet([('Answer', QuestionFormAnswer)])
+            h = handler.XmlHandler(answer_rs, connection)
+            value = connection.get_utf8_value(value)
+            xml.sax.parseString(value, h)
+            self.answers.append(answer_rs)
+        else:
+            BaseAutoResultElement.endElement(self, name, value, connection)
+
 
 class Assignment(BaseAutoResultElement):
     """
     Class to extract an Assignment structure from a response (used in
     ResultSet)
-    
-    Will have attributes named as per the Developer Guide, 
+
+    Will have attributes named as per the Developer Guide,
     e.g. AssignmentId, WorkerId, HITId, Answer, etc
     """
 
@@ -872,7 +986,7 @@ class Assignment(BaseAutoResultElement):
     def endElement(self, name, value, connection):
         # the answer consists of embedded XML, so it needs to be parsed independantly
         if name == 'Answer':
-            answer_rs = ResultSet([('Answer', QuestionFormAnswer),])
+            answer_rs = ResultSet([('Answer', QuestionFormAnswer)])
             h = handler.XmlHandler(answer_rs, connection)
             value = connection.get_utf8_value(value)
             xml.sax.parseString(value, h)
@@ -880,11 +994,12 @@ class Assignment(BaseAutoResultElement):
         else:
             BaseAutoResultElement.endElement(self, name, value, connection)
 
+
 class QuestionFormAnswer(BaseAutoResultElement):
     """
     Class to extract Answers from inside the embedded XML
     QuestionFormAnswers element inside the Answer element which is
-    part of the Assignment structure
+    part of the Assignment and QualificationRequest structures
 
     A QuestionFormAnswers element contains an Answer element for each
     question in the HIT or Qualification test for which the Worker
@@ -909,4 +1024,4 @@ class QuestionFormAnswer(BaseAutoResultElement):
         if name == 'QuestionIdentifier':
             self.qid = value
         elif name in ['FreeText', 'SelectionIdentifier', 'OtherSelectionText'] and self.qid:
-            self.fields.append( value )
+            self.fields.append(value)
