@@ -71,6 +71,33 @@ class SNSConnection(AWSQueryConnection):
                                     security_token=security_token,
                                     validate_certs=validate_certs)
 
+    def _build_dict_as_list_params(self, params, dictionary, name):
+      """
+            Serialize a parameter 'name' which value is a 'dictionary' into a list of parameters.
+
+            See: http://docs.aws.amazon.com/sns/latest/api/API_SetPlatformApplicationAttributes.html
+            For example::
+
+                dictionary = {'PlatformPrincipal': 'foo', 'PlatformCredential': 'bar'}
+                name = 'Attributes'
+
+            would result in params dict being populated with:
+                Attributes.entry.1.key    = PlatformPrincipal
+                Attributes.entry.1.value  = foo
+                Attributes.entry.2.key    = PlatformCredential
+                Attributes.entry.2.value  = bar
+
+      :param params: the resulting parameters will be added to this dict
+      :param dictionary: dict - value of the serialized parameter
+      :param name: name of the serialized parameter
+      """
+      items = sorted(dictionary.items(), key=lambda x:x[0])
+      for kv, index in zip(items, range(1, len(items)+1)):
+        key, value = kv
+        prefix = '%s.entry.%s' % (name, index)
+        params['%s.key' % prefix] = key
+        params['%s.value' % prefix] = value
+
     def _required_auth_capability(self):
         return ['hmac-v4']
 
@@ -182,8 +209,8 @@ class SNSConnection(AWSQueryConnection):
         params = {'TopicArn': topic}
         return self._make_request('DeleteTopic', params, '/', 'GET')
 
-    def publish(self, topic=None, message=None, subject=None,
-                target_arn=None):
+    def publish(self, topic=None, message=None, subject=None, target_arn=None,
+                message_structure=None):
         """
         Get properties of a Topic
 
@@ -195,12 +222,20 @@ class SNSConnection(AWSQueryConnection):
                         Messages must be UTF-8 encoded strings and
                         be at most 4KB in size.
 
+        :type message_structure: string
+        :param message_structure: Optional parameter. If left as ``None``,
+                                  plain text will be sent. If set to ``json``,
+                                  your message should be a JSON string that
+                                  matches the structure described at
+                                  http://docs.aws.amazon.com/sns/latest/dg/PublishTopic.html#sns-message-formatting-by-protocol
+
         :type subject: string
         :param subject: Optional parameter to be used as the "Subject"
                         line of the email notifications.
 
         :type target_arn: string
-        :param target_arn:
+        :param target_arn: Optional parameter for either TopicArn or
+                           EndpointArn, but not both.
 
         """
         if message is None:
@@ -215,7 +250,9 @@ class SNSConnection(AWSQueryConnection):
             params['TopicArn'] = topic
         if target_arn is not None:
             params['TargetArn'] = target_arn
-        return self._make_request('Publish', params)
+        if message_structure is not None:
+            params['MessageStructure'] = message_structure
+        return self._make_request('Publish', params, '/', 'POST')
 
     def subscribe(self, topic, protocol, endpoint):
         """
@@ -227,7 +264,7 @@ class SNSConnection(AWSQueryConnection):
         :type protocol: string
         :param protocol: The protocol used to communicate with
                          the subscriber.  Current choices are:
-                         email|email-json|http|https|sqs
+                         email|email-json|http|https|sqs|sms
 
         :type endpoint: string
         :param endpoint: The location of the endpoint for
@@ -237,6 +274,7 @@ class SNSConnection(AWSQueryConnection):
                          * For http, this would be a URL beginning with http
                          * For https, this would be a URL beginning with https
                          * For sqs, this would be the ARN of an SQS Queue
+                         * For sms, this would be a phone number of an SMS-enabled device 
         """
         params = {'TopicArn': topic,
                   'Protocol': protocol,
@@ -406,7 +444,7 @@ class SNSConnection(AWSQueryConnection):
         if platform is not None:
             params['Platform'] = platform
         if attributes is not None:
-            params['Attributes'] = attributes
+            self._build_dict_as_list_params(params, attributes, 'Attributes')
         return self._make_request(action='CreatePlatformApplication',
                                   params=params)
 
@@ -453,7 +491,7 @@ class SNSConnection(AWSQueryConnection):
         if platform_application_arn is not None:
             params['PlatformApplicationArn'] = platform_application_arn
         if attributes is not None:
-            params['Attributes'] = attributes
+            self._build_dict_as_list_params(params, attributes, 'Attributes')
         return self._make_request(action='SetPlatformApplicationAttributes',
                                   params=params)
 
@@ -601,7 +639,7 @@ class SNSConnection(AWSQueryConnection):
         if custom_user_data is not None:
             params['CustomUserData'] = custom_user_data
         if attributes is not None:
-            params['Attributes'] = attributes
+            self._build_dict_as_list_params(params, attributes, 'Attributes')
         return self._make_request(action='CreatePlatformEndpoint',
                                   params=params)
 
@@ -654,7 +692,7 @@ class SNSConnection(AWSQueryConnection):
         if endpoint_arn is not None:
             params['EndpointArn'] = endpoint_arn
         if attributes is not None:
-            params['Attributes'] = attributes
+            self._build_dict_as_list_params(params, attributes, 'Attributes')
         return self._make_request(action='SetEndpointAttributes',
                                   params=params)
 
