@@ -32,6 +32,8 @@ import base64
 import re
 
 from boto.utils import compute_md5
+from boto.utils import find_matching_headers
+from boto.utils import merge_headers_by_name
 from boto.s3.prefix import Prefix
 
 try:
@@ -99,12 +101,14 @@ class MockKey(object):
     def _handle_headers(self, headers):
         if not headers:
             return
-        if 'Content-Encoding' in headers:
-            self.content_encoding = headers['Content-Encoding']
-        if 'Content-Type' in headers:
-            self.content_type = headers['Content-Type']
-        if 'Content-Language' in headers:
-            self.content_language = headers['Content-Language']
+        if find_matching_headers('Content-Encoding', headers):
+            self.content_encoding = merge_headers_by_name('Content-Encoding',
+                                                          headers)
+        if find_matching_headers('Content-Type', headers):
+            self.content_type = merge_headers_by_name('Content-Type', headers)
+        if find_matching_headers('Content-Language', headers):
+            self.content_language = merge_headers_by_name('Content-Language',
+                                                          headers)
 
     # Simplistic partial implementation for headers: Just supports range GETs
     # of flavor 'Range: bytes=xyz-'.
