@@ -27,10 +27,10 @@ Unit tests for subscribing SQS queues to SNS topics.
 
 import hashlib
 import time
-import json
 
 from tests.unit import unittest
 
+from boto.compat import json
 from boto.sqs.connection import SQSConnection
 from boto.sns.connection import SNSConnection
 
