@@ -99,6 +99,79 @@ class TestSigV4Handler(unittest.TestCase):
         canonical_uri = auth.canonical_uri(request)
         self.assertEqual(canonical_uri, '/x/x.html')
 
+    def test_credential_scope(self):
+        # test the AWS standard regions IAM endpoint
+        auth = HmacAuthV4Handler('iam.amazonaws.com',
+                                 Mock(), self.provider)
+        request = HTTPRequest(
+            'POST', 'https', 'iam.amazonaws.com', 443,
+            '/', '/',
+            {'Action': 'ListAccountAliases', 'Version': '2010-05-08'},
+            {
+                'Content-Length': '44',
+                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
+                'X-Amz-Date': '20130808T013210Z'
+            },
+            'Action=ListAccountAliases&Version=2010-05-08')
+        credential_scope = auth.credential_scope(request)
+        region_name = credential_scope.split('/')[1]
+        self.assertEqual(region_name, 'us-east-1')
+
+        # test the AWS GovCloud region IAM endpoint
+        auth = HmacAuthV4Handler('iam.us-gov.amazonaws.com',
+                                 Mock(), self.provider)
+        request = HTTPRequest(
+            'POST', 'https', 'iam.us-gov.amazonaws.com', 443,
+            '/', '/',
+            {'Action': 'ListAccountAliases', 'Version': '2010-05-08'},
+            {
+                'Content-Length': '44',
+                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
+                'X-Amz-Date': '20130808T013210Z'
+            },
+            'Action=ListAccountAliases&Version=2010-05-08')
+        credential_scope = auth.credential_scope(request)
+        region_name = credential_scope.split('/')[1]
+        self.assertEqual(region_name, 'us-gov-west-1')
+
+        # iam.us-west-1.amazonaws.com does not exist however this
+        # covers the remaining region_name control structure for a
+        # different region name
+        auth = HmacAuthV4Handler('iam.us-west-1.amazonaws.com',
+                                 Mock(), self.provider)
+        request = HTTPRequest(
+            'POST', 'https', 'iam.us-west-1.amazonaws.com', 443,
+            '/', '/',
+            {'Action': 'ListAccountAliases', 'Version': '2010-05-08'},
+            {
+                'Content-Length': '44',
+                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
+                'X-Amz-Date': '20130808T013210Z'
+            },
+            'Action=ListAccountAliases&Version=2010-05-08')
+        credential_scope = auth.credential_scope(request)
+        region_name = credential_scope.split('/')[1]
+        self.assertEqual(region_name, 'us-west-1')
+
+        # Test connections to custom locations, e.g. localhost:8080
+        auth = HmacAuthV4Handler('localhost', Mock(), self.provider,
+                                 service_name='iam')
+
+        request = HTTPRequest(
+            'POST', 'http', 'localhost', 8080,
+            '/', '/',
+            {'Action': 'ListAccountAliases', 'Version': '2010-05-08'},
+            {
+                'Content-Length': '44',
+                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
+                'X-Amz-Date': '20130808T013210Z'
+            },
+            'Action=ListAccountAliases&Version=2010-05-08')
+        credential_scope = auth.credential_scope(request)
+        timestamp, region, service, v = credential_scope.split('/')
+        self.assertEqual(region, 'localhost')
+        self.assertEqual(service, 'iam')
+
     def test_headers_to_sign(self):
         auth = HmacAuthV4Handler('glacier.us-east-1.amazonaws.com',
                                  Mock(), self.provider)
