@@ -29,6 +29,7 @@ from tests.unit import AWSMockServiceTestCase
 from boto.ec2.autoscale import AutoScaleConnection
 from boto.ec2.autoscale.group import AutoScalingGroup
 from boto.ec2.autoscale.policy import ScalingPolicy
+from boto.ec2.autoscale.tag import Tag
 
 from boto.ec2.blockdevicemapping import EBSBlockDeviceType, BlockDeviceMapping
 
@@ -66,6 +67,29 @@ class TestAutoScaleGroup(AWSMockServiceTestCase):
             'TerminationPolicies.member.2': 'OldestLaunchConfiguration',
         }, ignore_params_values=['Version'])
 
+
+class TestAutoScaleGroupHonorCooldown(AWSMockServiceTestCase):
+    connection_class = AutoScaleConnection
+
+    def default_body(self):
+        return """
+            <SetDesiredCapacityResponse>
+              <ResponseMetadata>
+                <RequestId>9fb7e2db-6998-11e2-a985-57c82EXAMPLE</RequestId>
+              </ResponseMetadata>
+            </SetDesiredCapacityResponse>
+        """
+
+    def test_honor_cooldown(self):
+        self.set_http_response(status_code=200)
+        self.service_connection.set_desired_capacity('foo', 10, True)
+        self.assert_request_parameters({
+            'Action': 'SetDesiredCapacity',
+            'AutoScalingGroupName': 'foo',
+            'DesiredCapacity': 10,
+            'HonorCooldown': 'true',
+        }, ignore_params_values=['Version'])
+
 class TestScheduledGroup(AWSMockServiceTestCase):
     connection_class = AutoScaleConnection
 
@@ -226,19 +250,20 @@ class TestLaunchConfiguration(AWSMockServiceTestCase):
                 instance_type = 'm1.large',
                 security_groups = ['group1', 'group2'],
                 spot_price='price',
-                block_device_mappings = [bdm]
+                block_device_mappings = [bdm],
+                associate_public_ip_address = True
                 )
 
         response = self.service_connection.create_launch_configuration(lc)
 
         self.assert_request_parameters({
             'Action': 'CreateLaunchConfiguration',
-            'BlockDeviceMapping.1.DeviceName': '/dev/sdf',
-            'BlockDeviceMapping.1.Ebs.DeleteOnTermination': 'false',
-            'BlockDeviceMapping.1.Ebs.SnapshotId': 'snap-12345',
-            'BlockDeviceMapping.2.DeviceName': '/dev/sdg',
-            'BlockDeviceMapping.2.Ebs.DeleteOnTermination': 'false',
-            'BlockDeviceMapping.2.Ebs.SnapshotId': 'snap-12346',
+            'BlockDeviceMappings.member.1.DeviceName': '/dev/sdf',
+            'BlockDeviceMappings.member.1.Ebs.DeleteOnTermination': 'false',
+            'BlockDeviceMappings.member.1.Ebs.SnapshotId': 'snap-12345',
+            'BlockDeviceMappings.member.2.DeviceName': '/dev/sdg',
+            'BlockDeviceMappings.member.2.Ebs.DeleteOnTermination': 'false',
+            'BlockDeviceMappings.member.2.Ebs.SnapshotId': 'snap-12346',
             'EbsOptimized': 'false',
             'LaunchConfigurationName': 'launch_config',
             'ImageId': '123456',
@@ -247,6 +272,7 @@ class TestLaunchConfiguration(AWSMockServiceTestCase):
             'SecurityGroups.member.1': 'group1',
             'SecurityGroups.member.2': 'group2',
             'SpotPrice': 'price',
+            'AssociatePublicIpAddress' : 'true'
         }, ignore_params_values=['Version'])
 
 
@@ -325,5 +351,131 @@ class TestCreateAutoScalePolicy(AWSMockServiceTestCase):
         }, ignore_params_values=['Version'])
 
 
+class TestPutNotificationConfiguration(AWSMockServiceTestCase):
+    connection_class = AutoScaleConnection
+
+    def setUp(self):
+        super(TestPutNotificationConfiguration, self).setUp()
+
+    def default_body(self):
+        return """
+            <PutNotificationConfigurationResponse>
+              <ResponseMetadata>
+                <RequestId>requestid</RequestId>
+              </ResponseMetadata>
+            </PutNotificationConfigurationResponse>
+        """
+
+    def test_autoscaling_group_put_notification_configuration(self):
+        self.set_http_response(status_code=200)
+        autoscale = AutoScalingGroup(
+            name='ana', launch_config='lauch_config',
+            min_size=1, max_size=2,
+            termination_policies=['OldestInstance', 'OldestLaunchConfiguration'])
+        self.service_connection.put_notification_configuration(autoscale, 'arn:aws:sns:us-east-1:19890506:AutoScaling-Up', ['autoscaling:EC2_INSTANCE_LAUNCH'])
+        self.assert_request_parameters({
+            'Action': 'PutNotificationConfiguration',
+            'AutoScalingGroupName': 'ana',
+            'NotificationTypes.member.1': 'autoscaling:EC2_INSTANCE_LAUNCH',
+            'TopicARN': 'arn:aws:sns:us-east-1:19890506:AutoScaling-Up',
+        }, ignore_params_values=['Version'])
+
+
+class TestDeleteNotificationConfiguration(AWSMockServiceTestCase):
+    connection_class = AutoScaleConnection
+
+    def setUp(self):
+        super(TestDeleteNotificationConfiguration, self).setUp()
+
+    def default_body(self):
+        return """
+            <DeleteNotificationConfigurationResponse>
+              <ResponseMetadata>
+                <RequestId>requestid</RequestId>
+              </ResponseMetadata>
+            </DeleteNotificationConfigurationResponse>
+        """
+
+    def test_autoscaling_group_put_notification_configuration(self):
+        self.set_http_response(status_code=200)
+        autoscale = AutoScalingGroup(
+            name='ana', launch_config='lauch_config',
+            min_size=1, max_size=2,
+            termination_policies=['OldestInstance', 'OldestLaunchConfiguration'])
+        self.service_connection.delete_notification_configuration(autoscale, 'arn:aws:sns:us-east-1:19890506:AutoScaling-Up')
+        self.assert_request_parameters({
+            'Action': 'DeleteNotificationConfiguration',
+            'AutoScalingGroupName': 'ana',
+            'TopicARN': 'arn:aws:sns:us-east-1:19890506:AutoScaling-Up',
+        }, ignore_params_values=['Version'])
+
+class TestAutoScalingTag(AWSMockServiceTestCase):
+    connection_class = AutoScaleConnection
+
+    def default_body(self):
+        return """
+        <CreateOrUpdateTagsResponse>
+            <ResponseMetadata>
+                <RequestId>requestId</RequestId>
+            </ResponseMetadata>
+        </CreateOrUpdateTagsResponse>
+        """
+
+    def test_create_or_update_tags(self):
+        self.set_http_response(status_code=200)
+
+        tags = [
+            Tag(
+                connection=self.service_connection,
+                key='alpha',
+                value='tango',
+                resource_id='sg-00000000',
+                resource_type='auto-scaling-group',
+                propagate_at_launch=True
+                ),
+            Tag(
+                connection=self.service_connection,
+                key='bravo',
+                value='sierra',
+                resource_id='sg-00000000',
+                resource_type='auto-scaling-group',
+                propagate_at_launch=False
+                )]
+               
+
+        response = self.service_connection.create_or_update_tags(tags)
+
+        self.assert_request_parameters({
+            'Action': 'CreateOrUpdateTags',
+            'Tags.member.1.ResourceType': 'auto-scaling-group',
+            'Tags.member.1.ResourceId': 'sg-00000000',
+            'Tags.member.1.Key': 'alpha',
+            'Tags.member.1.Value': 'tango',
+            'Tags.member.1.PropagateAtLaunch': 'true',
+            'Tags.member.2.ResourceType': 'auto-scaling-group',
+            'Tags.member.2.ResourceId': 'sg-00000000',
+            'Tags.member.2.Key': 'bravo',
+            'Tags.member.2.Value': 'sierra',
+            'Tags.member.2.PropagateAtLaunch': 'false'
+        }, ignore_params_values=['Version'])
+
+    def test_endElement(self):
+        for i in [
+            ('Key', 'mykey', 'key'),
+            ('Value', 'myvalue', 'value'),
+            ('ResourceType', 'auto-scaling-group', 'resource_type'),
+            ('ResourceId', 'sg-01234567', 'resource_id'),
+            ('PropagateAtLaunch', 'true', 'propagate_at_launch')]:
+                self.check_tag_attributes_set(i[0], i[1], i[2])
+            
+             
+    def check_tag_attributes_set(self, name, value, attr):
+        tag = Tag()
+        tag.endElement(name, value, None)
+        if value == 'true':
+            self.assertEqual(getattr(tag, attr), True)
+        else:
+            self.assertEqual(getattr(tag, attr), value)
+
 if __name__ == '__main__':
     unittest.main()
