@@ -20,6 +20,8 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 #
+from __future__ import with_statement
+
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -32,6 +34,7 @@ from tests.unit import AWSMockServiceTestCase
 from boto.exception import BotoServerError
 from boto.s3.connection import S3Connection
 from boto.s3.bucket import Bucket
+from boto.s3.key import Key
 
 
 class TestS3Key(AWSMockServiceTestCase):
@@ -114,6 +117,51 @@ class TestS3KeyRetries(AWSMockServiceTestCase):
 
         self.assertTrue(k.should_retry.count, 1)
 
+    @mock.patch('time.sleep')
+    def test_502_bad_gateway(self, sleep_mock):
+        weird_timeout_body = "<Error><Code>BadGateway</Code></Error>"
+        self.set_http_response(status_code=502, body=weird_timeout_body)
+        b = Bucket(self.service_connection, 'mybucket')
+        k = b.new_key('test_failure')
+        fail_file = StringIO('This will pretend to be chunk-able.')
+
+        k.should_retry = counter(k.should_retry)
+        self.assertEqual(k.should_retry.count, 0)
+
+        with self.assertRaises(BotoServerError):
+            k.send_file(fail_file)
+
+        self.assertTrue(k.should_retry.count, 1)
+
+    @mock.patch('time.sleep')
+    def test_504_gateway_timeout(self, sleep_mock):
+        weird_timeout_body = "<Error><Code>GatewayTimeout</Code></Error>"
+        self.set_http_response(status_code=504, body=weird_timeout_body)
+        b = Bucket(self.service_connection, 'mybucket')
+        k = b.new_key('test_failure')
+        fail_file = StringIO('This will pretend to be chunk-able.')
+
+        k.should_retry = counter(k.should_retry)
+        self.assertEqual(k.should_retry.count, 0)
+
+        with self.assertRaises(BotoServerError):
+            k.send_file(fail_file)
+
+        self.assertTrue(k.should_retry.count, 1)
+
+
+class TestFileError(unittest.TestCase):
+    def test_file_error(self):
+        key = Key()
+
+        class CustomException(Exception): pass
+
+        key.get_contents_to_file = mock.Mock(
+            side_effect=CustomException('File blew up!'))
+
+        # Ensure our exception gets raised instead of a file or IO error
+        with self.assertRaises(CustomException):
+            key.get_contents_to_filename('foo.txt')
 
 if __name__ == '__main__':
     unittest.main()
